# -*- coding: utf-8 -*-
"""
WebResearcher command-line entry point.

This allows running WebResearcher as a module:
    python -m webresearcher "Your question here"
"""
from webresearcher.cli import main

if __name__ == '__main__':
    main()

